@@ -7,7 +7,7 @@ import logging
 import urllib
 from http.client import HTTPResponse
 from ssl import SSLContext
-from typing import Dict, Optional
+from typing import Dict, Optional, List, Any
 from urllib.error import HTTPError
 from urllib.request import Request, urlopen, OpenerDirector, ProxyHandler, HTTPSHandler
 
@@ -19,6 +19,11 @@ from .internal_utils import (
     get_user_agent,
 )
 from .response import AuditLogsResponse
+from slack_sdk.http_retry import default_retry_handlers
+from slack_sdk.http_retry.handler import RetryHandler
+from slack_sdk.http_retry.request import HttpRequest as RetryHttpRequest
+from slack_sdk.http_retry.response import HttpResponse as RetryHttpResponse
+from slack_sdk.http_retry.state import RetryState
 from ...proxy_env_variable_loader import load_http_proxy_from_env
 
 
@@ -32,6 +37,7 @@ class AuditLogsClient:
     base_url: str
     default_headers: Dict[str, str]
     logger: logging.Logger
+    retry_handlers: List[RetryHandler]
 
     def __init__(
         self,
@@ -44,6 +50,7 @@ class AuditLogsClient:
         user_agent_prefix: Optional[str] = None,
         user_agent_suffix: Optional[str] = None,
         logger: Optional[logging.Logger] = None,
+        retry_handlers: Optional[List[RetryHandler]] = None,
     ):
         """API client for Audit Logs API
         See https://api.slack.com/admins/audit-logs for more details
@@ -58,6 +65,7 @@ class AuditLogsClient:
             user_agent_prefix: Prefix for User-Agent header value
             user_agent_suffix: Suffix for User-Agent header value
             logger: Custom logger
+            retry_handlers: Retry handlers
         """
         self.token = token
         self.timeout = timeout
@@ -69,6 +77,9 @@ class AuditLogsClient:
             user_agent_prefix, user_agent_suffix
         )
         self.logger = logger if logger is not None else logging.getLogger(__name__)
+        self.retry_handlers = (
+            retry_handlers if retry_handlers is not None else default_retry_handlers()
+        )
 
         if self.proxy is None or len(self.proxy.strip()) == 0:
             env_variable = load_http_proxy_from_env(self.logger)
@@ -78,7 +89,7 @@ class AuditLogsClient:
     def schemas(
         self,
         *,
-        query_params: Optional[Dict[str, any]] = None,
+        query_params: Optional[Dict[str, Any]] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> AuditLogsResponse:
         """Returns information about the kind of objects which the Audit Logs API
@@ -100,7 +111,7 @@ class AuditLogsClient:
     def actions(
         self,
         *,
-        query_params: Optional[Dict[str, any]] = None,
+        query_params: Optional[Dict[str, Any]] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> AuditLogsResponse:
         """Returns information about the kind of actions that the Audit Logs API
@@ -129,7 +140,7 @@ class AuditLogsClient:
         action: Optional[str] = None,
         actor: Optional[str] = None,
         entity: Optional[str] = None,
-        additional_query_params: Optional[Dict[str, any]] = None,
+        additional_query_params: Optional[Dict[str, Any]] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> AuditLogsResponse:
         """This is the primary endpoint for retrieving actual audit events from your organization.
@@ -177,8 +188,8 @@ class AuditLogsClient:
         *,
         http_verb: str = "GET",
         path: str,
-        query_params: Optional[Dict[str, any]] = None,
-        body_params: Optional[Dict[str, any]] = None,
+        query_params: Optional[Dict[str, Any]] = None,
+        body_params: Optional[Dict[str, Any]] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> AuditLogsResponse:
         """Performs a Slack API request and returns the result."""
@@ -190,7 +201,7 @@ class AuditLogsClient:
         return self._perform_http_request(
             http_verb=http_verb,
             url=url,
-            body_params=body_params,
+            body=body_params,
             headers=_build_request_headers(
                 token=self.token,
                 default_headers=self.default_headers,
@@ -203,11 +214,11 @@ class AuditLogsClient:
         *,
         http_verb: str = "GET",
         url: str,
-        body_params: Optional[Dict[str, any]] = None,
+        body: Optional[Dict[str, Any]] = None,
         headers: Dict[str, str],
     ) -> AuditLogsResponse:
-        if body_params is not None:
-            body_params = json.dumps(body_params)
+        if body is not None:
+            body = json.dumps(body)
         headers["Content-Type"] = "application/json;charset=utf-8"
 
         if self.logger.level <= logging.DEBUG:
@@ -216,68 +227,158 @@ class AuditLogsClient:
                 for k, v in headers.items()
             }
             self.logger.debug(
-                f"Sending a request - url: {url}, body: {body_params}, headers: {headers_for_logging}"
+                f"Sending a request - url: {url}, body: {body}, headers: {headers_for_logging}"
             )
-        try:
-            opener: Optional[OpenerDirector] = None
-            # for security (BAN-B310)
-            if url.lower().startswith("http"):
-                req = Request(
-                    method=http_verb,
+
+        # NOTE: Intentionally ignore the `http_verb` here
+        # Slack APIs accepts any API method requests with POST methods
+        req = Request(
+            method=http_verb,
+            url=url,
+            data=body.encode("utf-8") if body is not None else None,
+            headers=headers,
+        )
+        resp = None
+        last_error = None
+
+        retry_state = RetryState()
+        counter_for_safety = 0
+        while counter_for_safety < 100:
+            counter_for_safety += 1
+            # If this is a retry, the next try started here. We can reset the flag.
+            retry_state.next_attempt_requested = False
+
+            try:
+                resp = self._perform_http_request_internal(url, req)
+                # The resp is a 200 OK response
+                return resp
+
+            except HTTPError as e:
+                # read the response body here
+                charset = e.headers.get_content_charset() or "utf-8"
+                response_body: str = e.read().decode(charset)
+                resp = AuditLogsResponse(
                     url=url,
-                    data=body_params.encode("utf-8")
-                    if body_params is not None
+                    status_code=e.code,
+                    raw_body=response_body,
+                    headers=dict(e.headers.items()),
+                )
+                if e.code == 429:
+                    # for backward-compatibility with WebClient (v.2.5.0 or older)
+                    if (
+                        "retry-after" not in resp.headers
+                        and "Retry-After" in resp.headers
+                    ):
+                        resp.headers["retry-after"] = resp.headers["Retry-After"]
+                    if (
+                        "Retry-After" not in resp.headers
+                        and "retry-after" in resp.headers
+                    ):
+                        resp.headers["Retry-After"] = resp.headers["retry-after"]
+                _debug_log_response(self.logger, resp)
+
+                # Try to find a retry handler for this error
+                retry_request = RetryHttpRequest.from_urllib_http_request(req)
+                retry_response = RetryHttpResponse(
+                    status_code=e.code,
+                    headers={k: [v] for k, v in e.headers.items()},
+                    data=response_body.encode("utf-8")
+                    if response_body is not None
                     else None,
-                    headers=headers,
                 )
-                if self.proxy is not None:
-                    if isinstance(self.proxy, str):
-                        opener = urllib.request.build_opener(
-                            ProxyHandler({"http": self.proxy, "https": self.proxy}),
-                            HTTPSHandler(context=self.ssl),
+                for handler in self.retry_handlers:
+                    if handler.can_retry(
+                        state=retry_state,
+                        request=retry_request,
+                        response=retry_response,
+                        error=e,
+                    ):
+                        if self.logger.level <= logging.DEBUG:
+                            self.logger.info(
+                                f"A retry handler found: {type(handler).__name__} for {req.method} {req.full_url} - {e}"
+                            )
+                        handler.prepare_for_next_attempt(
+                            state=retry_state,
+                            request=retry_request,
+                            response=retry_response,
+                            error=e,
                         )
-                    else:
-                        raise SlackRequestError(
-                            f"Invalid proxy detected: {self.proxy} must be a str value"
-                        )
-            else:
-                raise SlackRequestError(f"Invalid URL detected: {url}")
+                        break
 
-            # NOTE: BAN-B310 is already checked above
-            resp: Optional[HTTPResponse] = None
-            if opener:
-                resp = opener.open(req, timeout=self.timeout)  # skipcq: BAN-B310
-            else:
-                resp = urlopen(  # skipcq: BAN-B310
-                    req, context=self.ssl, timeout=self.timeout
+                if retry_state.next_attempt_requested is False:
+                    return resp
+
+            except Exception as err:
+                last_error = err
+                self.logger.error(
+                    f"Failed to send a request to Slack API server: {err}"
                 )
-            charset: str = resp.headers.get_content_charset() or "utf-8"
-            response_body: str = resp.read().decode(charset)
-            resp = AuditLogsResponse(
-                url=url,
-                status_code=resp.status,
-                raw_body=response_body,
-                headers=resp.headers,
-            )
-            _debug_log_response(self.logger, resp)
-            return resp
 
-        except HTTPError as e:
-            # read the response body here
-            charset = e.headers.get_content_charset() or "utf-8"
-            body_params: str = e.read().decode(charset)
-            resp = AuditLogsResponse(
-                url=url,
-                status_code=e.code,
-                raw_body=body_params,
-                headers=e.headers,
-            )
-            if e.code == 429:
-                # for backward-compatibility with WebClient (v.2.5.0 or older)
-                resp.headers["Retry-After"] = resp.headers["retry-after"]
-            _debug_log_response(self.logger, resp)
-            return resp
+                # Try to find a retry handler for this error
+                retry_request = RetryHttpRequest.from_urllib_http_request(req)
+                for handler in self.retry_handlers:
+                    if handler.can_retry(
+                        state=retry_state,
+                        request=retry_request,
+                        response=None,
+                        error=err,
+                    ):
+                        if self.logger.level <= logging.DEBUG:
+                            self.logger.info(
+                                f"A retry handler found: {type(handler).__name__} for {req.method} {req.full_url} - {err}"
+                            )
+                        handler.prepare_for_next_attempt(
+                            state=retry_state,
+                            request=retry_request,
+                            response=None,
+                            error=err,
+                        )
+                        self.logger.info(
+                            f"Going to retry the same request: {req.method} {req.full_url}"
+                        )
+                        break
 
-        except Exception as err:
-            self.logger.error(f"Failed to send a request to Slack API server: {err}")
-            raise err
+                if retry_state.next_attempt_requested is False:
+                    raise err
+
+        if resp is not None:
+            return resp
+        raise last_error
+
+    def _perform_http_request_internal(
+        self, url: str, req: Request
+    ) -> AuditLogsResponse:
+        opener: Optional[OpenerDirector] = None
+        # for security (BAN-B310)
+        if url.lower().startswith("http"):
+            if self.proxy is not None:
+                if isinstance(self.proxy, str):
+                    opener = urllib.request.build_opener(
+                        ProxyHandler({"http": self.proxy, "https": self.proxy}),
+                        HTTPSHandler(context=self.ssl),
+                    )
+                else:
+                    raise SlackRequestError(
+                        f"Invalid proxy detected: {self.proxy} must be a str value"
+                    )
+        else:
+            raise SlackRequestError(f"Invalid URL detected: {url}")
+
+        # NOTE: BAN-B310 is already checked above
+        http_resp: Optional[HTTPResponse] = None
+        if opener:
+            http_resp = opener.open(req, timeout=self.timeout)  # skipcq: BAN-B310
+        else:
+            http_resp = urlopen(  # skipcq: BAN-B310
+                req, context=self.ssl, timeout=self.timeout
+            )
+        charset: str = http_resp.headers.get_content_charset() or "utf-8"
+        response_body: str = http_resp.read().decode(charset)
+        resp = AuditLogsResponse(
+            url=url,
+            status_code=http_resp.status,
+            raw_body=response_body,
+            headers=http_resp.headers,
+        )
+        _debug_log_response(self.logger, resp)
+        return resp
