@@ -1,5 +1,7 @@
+import glob
 import json
 import logging
+import os
 from logging import Logger
 from pathlib import Path
 from typing import Optional, Union
@@ -37,6 +39,9 @@ class FileInstallationStore(InstallationStore, AsyncInstallationStore):
     async def async_save(self, installation: Installation):
         return self.save(installation)
 
+    async def async_save_bot(self, bot: Bot):
+        return self.save_bot(bot)
+
     def save(self, installation: Installation):
         none = "none"
         e_id = installation.enterprise_id or none
@@ -44,15 +49,11 @@ class FileInstallationStore(InstallationStore, AsyncInstallationStore):
         team_installation_dir = f"{self.base_dir}/{e_id}-{t_id}"
         self._mkdir(team_installation_dir)
 
+        self.save_bot(installation.to_bot())
+
         if self.historical_data_enabled:
             history_version: str = str(installation.installed_at)
 
-            entity: str = json.dumps(installation.to_bot().__dict__)
-            with open(f"{team_installation_dir}/bot-latest", "w") as f:
-                f.write(entity)
-            with open(f"{team_installation_dir}/bot-{history_version}", "w") as f:
-                f.write(entity)
-
             # per workspace
             entity: str = json.dumps(installation.__dict__)
             with open(f"{team_installation_dir}/installer-latest", "w") as f:
@@ -71,16 +72,32 @@ class FileInstallationStore(InstallationStore, AsyncInstallationStore):
                 f.write(entity)
 
         else:
-            with open(f"{team_installation_dir}/bot-latest", "w") as f:
-                entity: str = json.dumps(installation.to_bot().__dict__)
-                f.write(entity)
-
             u_id = installation.user_id or none
             installer_filepath = f"{team_installation_dir}/installer-{u_id}-latest"
             with open(installer_filepath, "w") as f:
                 entity: str = json.dumps(installation.__dict__)
                 f.write(entity)
 
+    def save_bot(self, bot: Bot):
+        none = "none"
+        e_id = bot.enterprise_id or none
+        t_id = bot.team_id or none
+        team_installation_dir = f"{self.base_dir}/{e_id}-{t_id}"
+        self._mkdir(team_installation_dir)
+
+        if self.historical_data_enabled:
+            history_version: str = str(bot.installed_at)
+
+            entity: str = json.dumps(bot.__dict__)
+            with open(f"{team_installation_dir}/bot-latest", "w") as f:
+                f.write(entity)
+            with open(f"{team_installation_dir}/bot-{history_version}", "w") as f:
+                f.write(entity)
+        else:
+            with open(f"{team_installation_dir}/bot-latest", "w") as f:
+                entity: str = json.dumps(bot.__dict__)
+                f.write(entity)
+
     async def async_find_bot(
         self,
         *,
@@ -112,8 +129,10 @@ class FileInstallationStore(InstallationStore, AsyncInstallationStore):
                 data = json.loads(f.read())
                 return Bot(**data)
         except FileNotFoundError as e:
-            message = f"Failed to find bot installation data for enterprise: {e_id}, team: {t_id}: {e}"
-            self.logger.warning(message)
+            message = (
+                f"Installation data missing for enterprise: {e_id}, team: {t_id}: {e}"
+            )
+            self.logger.debug(message)
             return None
 
     async def async_find_installation(
@@ -155,10 +174,61 @@ class FileInstallationStore(InstallationStore, AsyncInstallationStore):
                 data = json.loads(f.read())
                 return Installation(**data)
         except FileNotFoundError as e:
-            message = f"Failed to find an installation data for enterprise: {e_id}, team: {t_id}: {e}"
-            self.logger.warning(message)
+            message = (
+                f"Installation data missing for enterprise: {e_id}, team: {t_id}: {e}"
+            )
+            self.logger.debug(message)
             return None
 
+    async def async_delete_bot(
+        self, *, enterprise_id: Optional[str], team_id: Optional[str]
+    ) -> None:
+        return self.delete_bot(enterprise_id=enterprise_id, team_id=team_id)
+
+    def delete_bot(
+        self, *, enterprise_id: Optional[str], team_id: Optional[str]
+    ) -> None:
+        none = "none"
+        e_id = enterprise_id or none
+        t_id = team_id or none
+        filepath_glob = f"{self.base_dir}/{e_id}-{t_id}/bot-*"
+        self._delete_by_glob(e_id, t_id, filepath_glob)
+
+    async def async_delete_installation(
+        self,
+        *,
+        enterprise_id: Optional[str],
+        team_id: Optional[str],
+        user_id: Optional[str] = None,
+    ) -> None:
+        return self.delete_installation(
+            enterprise_id=enterprise_id, team_id=team_id, user_id=user_id
+        )
+
+    def delete_installation(
+        self,
+        *,
+        enterprise_id: Optional[str],
+        team_id: Optional[str],
+        user_id: Optional[str] = None,
+    ) -> None:
+        none = "none"
+        e_id = enterprise_id or none
+        t_id = team_id or none
+        if user_id is not None:
+            filepath_glob = f"{self.base_dir}/{e_id}-{t_id}/installer-{user_id}-*"
+        else:
+            filepath_glob = f"{self.base_dir}/{e_id}-{t_id}/installer-*"
+        self._delete_by_glob(e_id, t_id, filepath_glob)
+
+    def _delete_by_glob(self, e_id: str, t_id: str, filepath_glob: str):
+        for filepath in glob.glob(filepath_glob):
+            try:
+                os.remove(filepath)
+            except FileNotFoundError as e:
+                message = f"Failed to delete installation data for enterprise: {e_id}, team: {t_id}: {e}"
+                self.logger.warning(message)
+
     @staticmethod
     def _mkdir(path: Union[str, Path]):
         if isinstance(path, str):
