@@ -1,9 +1,15 @@
+"""SCIM API is a set of APIs for provisioning and managing user accounts and groups.
+SCIM is used by Single Sign-On (SSO) services and identity providers to manage people across a variety of tools,
+including Slack.
+
+Refer to https://slack.dev/python-slack-sdk/scim/ for details.
+"""
 import json
 import logging
 import urllib
 from http.client import HTTPResponse
 from ssl import SSLContext
-from typing import Dict, Optional, Union, Any
+from typing import Dict, Optional, Union, Any, List
 from urllib.error import HTTPError
 from urllib.parse import quote
 from urllib.request import Request, urlopen, OpenerDirector, ProxyHandler, HTTPSHandler
@@ -33,6 +39,13 @@ from .response import (
 )
 from .user import User
 from .group import Group
+
+from slack_sdk.http_retry import default_retry_handlers
+from slack_sdk.http_retry.handler import RetryHandler
+from slack_sdk.http_retry.request import HttpRequest as RetryHttpRequest
+from slack_sdk.http_retry.response import HttpResponse as RetryHttpResponse
+from slack_sdk.http_retry.state import RetryState
+
 from ...proxy_env_variable_loader import load_http_proxy_from_env
 
 
@@ -46,6 +59,7 @@ class SCIMClient:
     base_url: str
     default_headers: Dict[str, str]
     logger: logging.Logger
+    retry_handlers: List[RetryHandler]
 
     def __init__(
         self,
@@ -58,18 +72,22 @@ class SCIMClient:
         user_agent_prefix: Optional[str] = None,
         user_agent_suffix: Optional[str] = None,
         logger: Optional[logging.Logger] = None,
+        retry_handlers: Optional[List[RetryHandler]] = None,
     ):
         """API client for SCIM API
         See https://api.slack.com/scim for more details
-        :param token: An admin user's token, which starts with xoxp-
-        :param timeout: request timeout (in seconds)
-        :param ssl: ssl.SSLContext to use for requests
-        :param proxy: proxy URL (e.g., localhost:9000, http://localhost:9000)
-        :param base_url: the base URL for API calls
-        :param default_headers: request headers to add to all requests
-        :param user_agent_prefix: prefix for User-Agent header value
-        :param user_agent_suffix: suffix for User-Agent header value
-        :param logger: custom logger
+
+        Args:
+            token: An admin user's token, which starts with `xoxp-`
+            timeout: Request timeout (in seconds)
+            ssl: `ssl.SSLContext` to use for requests
+            proxy: Proxy URL (e.g., `localhost:9000`, `http://localhost:9000`)
+            base_url: The base URL for API calls
+            default_headers: Request headers to add to all requests
+            user_agent_prefix: Prefix for User-Agent header value
+            user_agent_suffix: Suffix for User-Agent header value
+            logger: Custom logger
+            retry_handlers: Retry handlers
         """
         self.token = token
         self.timeout = timeout
@@ -81,6 +99,9 @@ class SCIMClient:
             user_agent_prefix, user_agent_suffix
         )
         self.logger = logger if logger is not None else logging.getLogger(__name__)
+        self.retry_handlers = (
+            retry_handlers if retry_handlers is not None else default_retry_handlers()
+        )
 
         if self.proxy is None or len(self.proxy.strip()) == 0:
             env_variable = load_http_proxy_from_env(self.logger)
@@ -141,10 +162,11 @@ class SCIMClient:
         )
 
     def update_user(self, user: Union[Dict[str, Any], User]) -> UserUpdateResponse:
+        user_id = user.id if isinstance(user, User) else user["id"]
         return UserUpdateResponse(
             self.api_call(
                 http_verb="PUT",
-                path=f"Users/{quote(user.id)}",
+                path=f"Users/{quote(user_id)}",
                 body_params=user.to_dict()
                 if isinstance(user, User)
                 else _to_dict_without_not_given(user),
@@ -213,10 +235,11 @@ class SCIMClient:
         )
 
     def update_group(self, group: Union[Dict[str, Any], Group]) -> GroupUpdateResponse:
+        group_id = group.id if isinstance(group, Group) else group["id"]
         return GroupUpdateResponse(
             self.api_call(
                 http_verb="PUT",
-                path=f"Groups/{quote(group.id)}",
+                path=f"Groups/{quote(group_id)}",
                 body_params=group.to_dict()
                 if isinstance(group, Group)
                 else _to_dict_without_not_given(group),
@@ -251,7 +274,7 @@ class SCIMClient:
         return self._perform_http_request(
             http_verb=http_verb,
             url=url,
-            body_params=body_params,
+            body=body_params,
             headers=_build_request_headers(
                 token=self.token,
                 default_headers=self.default_headers,
@@ -264,13 +287,13 @@ class SCIMClient:
         *,
         http_verb: str = "GET",
         url: str,
-        body_params: Optional[Dict[str, Any]] = None,
+        body: Optional[Dict[str, Any]] = None,
         headers: Dict[str, str],
     ) -> SCIMResponse:
-        if body_params is not None:
-            if body_params.get("schemas") is None:
-                body_params["schemas"] = ["urn:scim:schemas:core:1.0"]
-            body_params = json.dumps(body_params)
+        if body is not None:
+            if body.get("schemas") is None:
+                body["schemas"] = ["urn:scim:schemas:core:1.0"]
+            body = json.dumps(body)
         headers["Content-Type"] = "application/json;charset=utf-8"
 
         if self.logger.level <= logging.DEBUG:
@@ -279,68 +302,156 @@ class SCIMClient:
                 for k, v in headers.items()
             }
             self.logger.debug(
-                f"Sending a request - {http_verb} url: {url}, body: {body_params}, headers: {headers_for_logging}"
+                f"Sending a request - {http_verb} url: {url}, body: {body}, headers: {headers_for_logging}"
             )
-        try:
-            opener: Optional[OpenerDirector] = None
-            # for security (BAN-B310)
-            if url.lower().startswith("http"):
-                req = Request(
-                    method=http_verb,
+
+        # NOTE: Intentionally ignore the `http_verb` here
+        # Slack APIs accepts any API method requests with POST methods
+        req = Request(
+            method=http_verb,
+            url=url,
+            data=body.encode("utf-8") if body is not None else None,
+            headers=headers,
+        )
+        resp = None
+        last_error = None
+
+        retry_state = RetryState()
+        counter_for_safety = 0
+        while counter_for_safety < 100:
+            counter_for_safety += 1
+            # If this is a retry, the next try started here. We can reset the flag.
+            retry_state.next_attempt_requested = False
+
+            try:
+                resp = self._perform_http_request_internal(url, req)
+                # The resp is a 200 OK response
+                return resp
+
+            except HTTPError as e:
+                # read the response body here
+                charset = e.headers.get_content_charset() or "utf-8"
+                response_body: str = e.read().decode(charset)
+                resp = SCIMResponse(
                     url=url,
-                    data=body_params.encode("utf-8")
-                    if body_params is not None
+                    status_code=e.code,
+                    raw_body=response_body,
+                    headers=dict(e.headers.items()),
+                )
+                if e.code == 429:
+                    # for backward-compatibility with WebClient (v.2.5.0 or older)
+                    if (
+                        "retry-after" not in resp.headers
+                        and "Retry-After" in resp.headers
+                    ):
+                        resp.headers["retry-after"] = resp.headers["Retry-After"]
+                    if (
+                        "Retry-After" not in resp.headers
+                        and "retry-after" in resp.headers
+                    ):
+                        resp.headers["Retry-After"] = resp.headers["retry-after"]
+                _debug_log_response(self.logger, resp)
+
+                # Try to find a retry handler for this error
+                retry_request = RetryHttpRequest.from_urllib_http_request(req)
+                retry_response = RetryHttpResponse(
+                    status_code=e.code,
+                    headers={k: [v] for k, v in e.headers.items()},
+                    data=response_body.encode("utf-8")
+                    if response_body is not None
                     else None,
-                    headers=headers,
                 )
-                if self.proxy is not None:
-                    if isinstance(self.proxy, str):
-                        opener = urllib.request.build_opener(
-                            ProxyHandler({"http": self.proxy, "https": self.proxy}),
-                            HTTPSHandler(context=self.ssl),
+                for handler in self.retry_handlers:
+                    if handler.can_retry(
+                        state=retry_state,
+                        request=retry_request,
+                        response=retry_response,
+                        error=e,
+                    ):
+                        if self.logger.level <= logging.DEBUG:
+                            self.logger.info(
+                                f"A retry handler found: {type(handler).__name__} for {req.method} {req.full_url} - {e}"
+                            )
+                        handler.prepare_for_next_attempt(
+                            state=retry_state,
+                            request=retry_request,
+                            response=retry_response,
+                            error=e,
                         )
-                    else:
-                        raise SlackRequestError(
-                            f"Invalid proxy detected: {self.proxy} must be a str value"
-                        )
-            else:
-                raise SlackRequestError(f"Invalid URL detected: {url}")
+                        break
 
-            # NOTE: BAN-B310 is already checked above
-            resp: Optional[HTTPResponse] = None
-            if opener:
-                resp = opener.open(req, timeout=self.timeout)  # skipcq: BAN-B310
-            else:
-                resp = urlopen(  # skipcq: BAN-B310
-                    req, context=self.ssl, timeout=self.timeout
+                if retry_state.next_attempt_requested is False:
+                    return resp
+
+            except Exception as err:
+                last_error = err
+                self.logger.error(
+                    f"Failed to send a request to Slack API server: {err}"
                 )
-            charset: str = resp.headers.get_content_charset() or "utf-8"
-            response_body: str = resp.read().decode(charset)
-            resp = SCIMResponse(
-                url=url,
-                status_code=resp.status,
-                raw_body=response_body,
-                headers=resp.headers,
-            )
-            _debug_log_response(self.logger, resp)
-            return resp
 
-        except HTTPError as e:
-            # read the response body here
-            charset = e.headers.get_content_charset() or "utf-8"
-            body_params: str = e.read().decode(charset)
-            resp = SCIMResponse(
-                url=url,
-                status_code=e.code,
-                raw_body=body_params,
-                headers=e.headers,
-            )
-            if e.code == 429:
-                # for backward-compatibility with WebClient (v.2.5.0 or older)
-                resp.headers["Retry-After"] = resp.headers["retry-after"]
-            _debug_log_response(self.logger, resp)
-            return resp
+                # Try to find a retry handler for this error
+                retry_request = RetryHttpRequest.from_urllib_http_request(req)
+                for handler in self.retry_handlers:
+                    if handler.can_retry(
+                        state=retry_state,
+                        request=retry_request,
+                        response=None,
+                        error=err,
+                    ):
+                        if self.logger.level <= logging.DEBUG:
+                            self.logger.info(
+                                f"A retry handler found: {type(handler).__name__} for {req.method} {req.full_url} - {err}"
+                            )
+                        handler.prepare_for_next_attempt(
+                            state=retry_state,
+                            request=retry_request,
+                            response=None,
+                            error=err,
+                        )
+                        self.logger.info(
+                            f"Going to retry the same request: {req.method} {req.full_url}"
+                        )
+                        break
 
-        except Exception as err:
-            self.logger.error(f"Failed to send a request to Slack API server: {err}")
-            raise err
+                if retry_state.next_attempt_requested is False:
+                    raise err
+
+        if resp is not None:
+            return resp
+        raise last_error
+
+    def _perform_http_request_internal(self, url: str, req: Request) -> SCIMResponse:
+        opener: Optional[OpenerDirector] = None
+        # for security (BAN-B310)
+        if url.lower().startswith("http"):
+            if self.proxy is not None:
+                if isinstance(self.proxy, str):
+                    opener = urllib.request.build_opener(
+                        ProxyHandler({"http": self.proxy, "https": self.proxy}),
+                        HTTPSHandler(context=self.ssl),
+                    )
+                else:
+                    raise SlackRequestError(
+                        f"Invalid proxy detected: {self.proxy} must be a str value"
+                    )
+        else:
+            raise SlackRequestError(f"Invalid URL detected: {url}")
+
+        # NOTE: BAN-B310 is already checked above
+        http_resp: Optional[HTTPResponse] = None
+        if opener:
+            http_resp = opener.open(req, timeout=self.timeout)  # skipcq: BAN-B310
+        else:
+            http_resp = urlopen(  # skipcq: BAN-B310
+                req, context=self.ssl, timeout=self.timeout
+            )
+        charset: str = http_resp.headers.get_content_charset() or "utf-8"
+        response_body: str = http_resp.read().decode(charset)
+        resp = SCIMResponse(
+            url=url,
+            status_code=http_resp.status,
+            raw_body=response_body,
+            headers=http_resp.headers,
+        )
+        _debug_log_response(self.logger, resp)
+        return resp
